﻿namespace BoasPraticas.CodigoBom
{
    public enum StatusDaConta
    {
        NaoRegistrado = 1,
        ClienteComum = 2,
        ClienteEspecial = 3,
        ClienteVIP = 4
    }

    public static class Constantes
    {
        public const int DESCONTO_MAXIMO_POR_FIDELIDADE = 5;
        public const decimal DESCONTO_CLIENTE_COMUM = 0.1m;
        public const decimal DESCONTO_CLIENTE_ESPECIAL = 0.3m;
        public const decimal DESCONTO_CLIENTE_VIP = 0.5m;
    }

    public class GerenciadorDeDescontos
    {
        private readonly ICalculoDescontoStatusContaFactory _descontoStatusConta;
        private readonly ICalculoDescontoFidelidade _descontoFidelidade;

        public GerenciadorDeDescontos(ICalculoDescontoStatusContaFactory descontoStatusConta, ICalculoDescontoFidelidade descontoFidelidade)
        {
            _descontoStatusConta = descontoStatusConta;
            _descontoFidelidade = descontoFidelidade;
        }

        public decimal AplicarDesconto(decimal preco, StatusDaConta statusDaConta, int tempoDeContaEmAnos)
        {

            decimal precoDepoisDoDesconto = 0;

            precoDepoisDoDesconto = _descontoStatusConta.GetCalculoDescontoStatusConta(statusDaConta).AplicarDesconto(preco);
            precoDepoisDoDesconto = _descontoFidelidade.AplicarDesconto(precoDepoisDoDesconto, tempoDeContaEmAnos);

            return precoDepoisDoDesconto;
        }
    }
}
